import matplotlib.pyplot as plt
import numpy as np
from matplotlib.colors import LinearSegmentedColormap
from matplotlib.backends.backend_qt5agg import FigureCanvasQTAgg as FigureCanvas
from matplotlib.patches import FancyBboxPatch, Rectangle, Circle


class VisualSVCanvas(FigureCanvas):
    def __init__(self, matrix, interval, chrom_list, start_list, end_list, SV_list, resolution, output_widget):
        self.fig, (self.ax_genome, self.ax_heatmap) = plt.subplots(2, 1, 
                                                                 gridspec_kw={'height_ratios': [1, 10]}, 
                                                                 figsize=(10, 10), sharex=True)
        super(VisualSVCanvas, self).__init__(self.fig)
        self.output_widget = output_widget
        self.interval = interval
        self.chrom_list = chrom_list
        self.start_list = start_list
        self.end_list = end_list
        self.resolution = resolution
        cdict = {
            'red': ((0.0, 0.0, 1.0), (1.0, 1.0, 1.0)),
            'green': ((0.0, 0.0, 1.0), (1.0, 0.0, 0.0)),
            'blue': ((0.0, 0.0, 1.0), (1.0, 0.0, 0.0))
        }
        cmap = LinearSegmentedColormap('Rd_Bl_Rd', cdict, 256)
        
        self.data = matrix
        vmax = np.percentile(matrix, 95)
        vmin = matrix.min()
        self.image = self.ax_heatmap.imshow(matrix, cmap=cmap, vmin=vmin, vmax=vmax)
        #self.ax_heatmap.axis('off')
        self.ax_heatmap.tick_params(left=False, bottom=False, labelleft=False, labelbottom=False)
        row = matrix.shape[0]
        # plot chrom boundary
        for i in range(len(interval)):
            if interval[i] != 0:
                self.ax_heatmap.plot([0, row], [interval[i], interval[i]],
                        color="black", linestyle='--', linewidth=2)
                self.ax_heatmap.plot([interval[i], interval[i]], [0, row],
                        color="black", linestyle='--', linewidth=2)
        # plot SV block
        block_x_start, block_y_start = SV_list[0], SV_list[2]
        block_width, block_height = SV_list[3]-SV_list[2], SV_list[1]-SV_list[0]
        if SV_list[0] == SV_list[1]:
            self.ax_heatmap.add_patch(Circle((block_y_start, block_x_start), radius=0.5,
                                     edgecolor='black', facecolor='none',
                                     linestyle='--', linewidth=2))
        else:
            self.ax_heatmap.add_patch(Rectangle((block_y_start, block_x_start), block_width, block_height, 
                                    edgecolor='black', facecolor='none', 
                                    linestyle='--', linewidth=2))
        # plot genome region
        start_pos = 0
        for k in range(len(start_list)):
            width = (end_list[k] - start_list[k]) // resolution
            self.ax_genome.add_patch(FancyBboxPatch((start_pos, 0), width, 1,
                                               boxstyle="round,pad=0.3", 
                                            facecolor=(245/255, 195/255, 66/255), edgecolor='black', alpha=0.7))
            self.ax_genome.text(start_pos + (start_list[k] + end_list[k]) // (2 * resolution), 0.5, chrom_list[k], ha='center', va='center')   
            self.ax_genome.text(start_pos, 1.05, f'{round(start_list[k]/1000000, 2)}', 
                    ha='left', va='bottom', fontsize=10, color='black')
            self.ax_genome.text(start_pos + width, 1.05, f'{round(end_list[k]/1000000, 2)}M', 
                    ha='right', va='bottom', fontsize=10, color='black')
            start_pos += width

        self.ax_heatmap.set_xlim(0, matrix.shape[1])
        self.ax_genome.set_xlim(0, matrix.shape[1])
        self.ax_genome.axis('off')
        self.ax_genome.set_position([0.1, 0.9, 0.8, 0.03])
        self.ax_heatmap.set_position([0.1, 0.1, 0.8, 0.8])
        # mouse_connect
        self.mpl_connect('scroll_event', self.on_scroll)
        self.mpl_connect('button_press_event', self.on_press)
        self.mpl_connect('motion_notify_event', self.on_move)
        self.mpl_connect('button_release_event', self.on_release)
        self.mpl_connect('button_press_event', self.on_click)
        # set press
        self.press = None
        self.draw()

    def on_scroll(self, event):
        x_left, x_right = self.ax_heatmap.get_xlim()
        y_bottom, y_top = self.ax_heatmap.get_ylim()
        # set zoom factor
        zoom_factor = 1.2

        if event.button == 'up':
            scale_factor = 1 / zoom_factor
        elif event.button == 'down':
            scale_factor = zoom_factor
        else:
            scale_factor = 1

        xdata, ydata = event.xdata, event.ydata
        new_width = (x_right - x_left) * scale_factor
        new_height = (y_top - y_bottom) * scale_factor

        x_left = xdata - (xdata - x_left) * scale_factor
        x_right = xdata + (x_right - xdata) * scale_factor
        y_bottom = ydata - (ydata - y_bottom) * scale_factor
        y_top = ydata + (y_top - ydata) * scale_factor

        self.ax_heatmap.set_xlim([x_left, x_right])
        self.ax_heatmap.set_ylim([y_bottom, y_top])

        self.draw()

    def on_press(self, event):
        if event.button == 1:
            if event.inaxes != self.ax_heatmap:
                return
            self.press = (event.xdata, event.ydata)
            if event.dblclick:
                self.on_click(event) 

    def on_move(self, event):
        if self.press is None:
            return
        if event.xdata is None or event.ydata is None:
            return
        x_left, x_right = self.ax_heatmap.get_xlim()
        y_bottom, y_top = self.ax_heatmap.get_ylim()
        dx = event.xdata - self.press[0]
        dy = event.ydata - self.press[1]
        self.ax_heatmap.set_xlim(x_left - dx, x_right - dx)
        self.ax_heatmap.set_ylim(y_bottom - dy, y_top - dy)
        self.press = (event.xdata, event.ydata)
        self.draw()

    def on_release(self, event):
        self.press = None

    def on_click(self, event):
        if event.inaxes != self.ax_heatmap:
            return
        x_click = int(event.xdata)
        y_click = int(event.ydata)
        output_text = ""
        if 0 <= x_click < self.data.shape[1] and 0 <= y_click < self.data.shape[0]:
            pixel_value = self.data[y_click, x_click]
            if len(self.interval) == 0:
                # intra
                cur_chrom = self.chrom_list[0]
                cur_start = self.start_list[0] + y_click * self.resolution
                cur_end = self.start_list[0] + x_click * self.resolution
                output_text += "position1: " + "\t"  + str(cur_chrom) + "\t" + str(cur_start) + "\n"
                output_text += "position2: " + "\t"  + str(cur_chrom) + "\t" + str(cur_end) + "\n"
            else:
                # inter
                if x_click <= self.interval[0] and y_click <= self.interval[0]:
                    # region_A
                    cur_start = self.start_list[0] + y_click * self.resolution
                    cur_end = self.start_list[0] + x_click * self.resolution
                    output_text += "position1: " + "\t"  + str(self.chrom_list[0]) + "\t" + str(cur_start) + "\n"
                    output_text += "position2: " + "\t"  + str(self.chrom_list[0]) + "\t" + str(cur_end) + "\n"
                elif x_click > self.interval[0] and y_click > self.interval[0]:
                    # region_B
                    cur_start = self.start_list[1] + y_click * self.resolution
                    cur_end = self.start_list[1] + x_click * self.resolution
                    output_text += "position1: " + "\t"  + str(self.chrom_list[0]) + "\t" + str(cur_start) + "\n"
                    output_text += "position2: " + "\t"  + str(self.chrom_list[0]) + "\t" + str(cur_end) + "\n"
                elif x_click <= self.interval[0] and y_click > self.interval[0]:
                    # region_C
                    cur_start = self.start_list[1] + (y_click-self.interval[0]) * self.resolution
                    cur_end = self.start_list[0] + x_click * self.resolution
                    output_text += "position1: " + "\t"  + str(self.chrom_list[0]) + "\t" + str(cur_start) + "\n"
                    output_text += "position2: " + "\t"  + str(self.chrom_list[1]) + "\t" + str(cur_end) + "\n"
                else:
                    # region_D
                    cur_start = self.start_list[0] + y_click * self.resolution
                    cur_end = self.start_list[1] + (x_click-self.interval[0]) * self.resolution
                    output_text += "position1: " + "\t"  + str(self.chrom_list[0]) + "\t" + str(cur_start) + "\n"
                    output_text += "position2: " + "\t"  + str(self.chrom_list[1]) + "\t" + str(cur_end) + "\n"
            output_text += "Count: " + "\t" + str(pixel_value)
            self.output_widget.setText(output_text)


def plot_current_SV(matrix, interval, chrom_list, start_list, end_list, SV_list, resolution, output_widget):
    canvas = VisualSVCanvas(matrix, interval, chrom_list, start_list, end_list, SV_list, resolution, output_widget)
    return canvas


def extra_SV_region_intra(clr, region):
    matrix = clr.matrix(balance=False).fetch(region)
    return matrix, []

def extra_SV_region_inter(clr, region1, region2):
    matrix1 = clr.matrix(balance=False).fetch(region1)
    matrix2 = clr.matrix(balance=False).fetch(region2)
    matrix_inter = clr.matrix(balance=False).fetch(region1, region2)
    # combine these matrix into one matrix
    n1 = matrix1.shape[0]
    n2 = matrix2.shape[0]
    final_matrix = np.zeros((n1 + n2, n1 + n2))
    final_matrix[:n1, :n1] = matrix1
    final_matrix[n1:, n1:] = matrix2
    final_matrix[:n1, n1:] = matrix_inter
    final_matrix[n1:, :n1] = matrix_inter.T
    return final_matrix, [n1]



import matplotlib.pyplot as plt
from matplotlib.colors import LinearSegmentedColormap
import numpy as np
from sklearn.isotonic import IsotonicRegression
from sklearn.linear_model import LinearRegression
import itertools, cooler
from matplotlib.backends.backend_qt5agg import FigureCanvasQTAgg as FigureCanvas
import decgr.correct

class MyFigureCanvas(FigureCanvas):
    def __init__(self, width, height, dpi, matrix, interval, orient, path):
        self.fig = plt.figure(figsize=(width, height), dpi=dpi)
        super(MyFigureCanvas, self).__init__(self.fig)
        self.vmax = np.percentile(matrix, 98)
        self.vmin = matrix.min()
        cdict = {
            'red': ((0.0, 0.0, 1.0), (1.0, 1.0, 1.0)),
            'green': ((0.0, 0.0, 1.0), (1.0, 0.0, 0.0)),
            'blue': ((0.0, 0.0, 1.0), (1.0, 0.0, 0.0))
        }
        self.cmap = LinearSegmentedColormap('Rd_Bl_Rd', cdict, 256)
        self.axes = self.fig.add_subplot(111)
        self.axes.axis('off')
        row = matrix.shape[0]
        M = matrix
        n = M.shape[0]
        # Create the rotation matrix
        t = np.array([[1, 0.5], [-1, 0.5]])
        A = np.dot(np.array([(i[1], i[0]) for i in itertools.product(range(n, -1, -1), range(0, n + 1, 1))]), t)

        # Plot the Heatmap ...
        x = A[:, 1].reshape(n + 1, n + 1)
        y = A[:, 0].reshape(n + 1, n + 1)
        y[y < 0] = -y[y < 0]
        self.axes.pcolormesh(x, y, np.flipud(M), vmin=self.vmin, vmax=self.vmax, cmap=self.cmap,
                      edgecolor='none', snap=True, linewidth=.01, rasterized=True)
        # plot boundary
        for i in range(len(interval)):
            if interval[i] != 0:
                self.axes.plot([x[(n - interval[i]), 0], x[(n - interval[i]), interval[i]]], [y[(n - interval[i]), 0], 0],
                        color="black", linestyle='--', linewidth=2)
                self.axes.plot([x[(n - interval[i]), n], x[(n - interval[i]), interval[i]]], [y[(n - interval[i]), n], 0],
                        color="black", linestyle='--', linewidth=2)
        self.axes.plot([0, x[0, 0]], [0, y[0, 0]], color="black", linewidth=2)
        self.axes.plot([x[0, 0], n], [y[0, 0], 0], color="black", linewidth=2)

        # plot CSV
        h = -10
        interval.append(n)
        for i in range(len(interval) - 1):
            if orient[i] == '+':
                pos1 = interval[i + 1]
                pos2 = interval[i]
            else:
                pos1 = interval[i]
                pos2 = interval[i + 1]
            textPos = interval[i] + (interval[i + 1] - interval[i]) // 2
            plt.text(textPos, h + 4, path[i],
                     fontdict={'family': 'Arial', 'size': 16, 'color': 'black', 'weight': 'bold'})
            self.axes.annotate("",
                        xy=[pos1, h],
                        xytext=[pos2, h],
                        size=10,
                        arrowprops=dict(color='gray', headwidth=20, width=10, headlength=10, ec="black", lw=1),
                        zorder=0)

        self.axes.axis('off')
        plt.ylim([-10, n])
        plt.subplots_adjust(0.1, 0.1, 0.9, 0.8)
        self.isshowing=False


def plot_assembly_matrix(matrix, interval, path, orient):
    canvas = MyFigureCanvas(width=5, height=4, dpi=100, matrix=matrix, interval=interval, orient=orient, path=path)
    return canvas


def joint_SV(path, orient, chrom, start, end, dict_matrix):
    genome = []
    SV_chrom = []
    for i in range(len(path)):
        if orient[i] == '+':
            fragment = list(range(start[i], end[i]))
            cur_chrom = [chrom[i]] * (end[i] - start[i])
            genome.extend(fragment)
            SV_chrom.extend(cur_chrom)
        else:
            fragment = list(range(end[i], start[i], -1))
            cur_chrom = [chrom[i]] * (end[i] - start[i])
            SV_chrom.extend(cur_chrom)
            genome.extend(fragment)
    # init_SV_matrix
    rows = len(genome)
    SV_matrix = np.full((rows, rows), 0.0)
    for i in range(len(genome)):
        for j in range(i, len(genome)):
            chrom_pair = SV_chrom[i] + '_' + SV_chrom[j]
            cur_matrix = dict_matrix[chrom_pair]
            SV_matrix[i][j] = cur_matrix[genome[i]][genome[j]]

    return SV_matrix


def get_start(path, start, end):
    cur_start = 0
    if len(path) > 1:
        for i in range(len(path)-1):
            # index = ord(path[i]) - 65
            cur_start += (end[i] - start[i])
    return cur_start


def linear_regression(SV_exp, global_exp):
    X = []
    Y = []
    Xi = []
    for i in sorted(SV_exp):
        if i in global_exp:
            Xi.append(i)
            X.append(global_exp[i])
            Y.append(SV_exp[i])
    X = np.r_[X]
    Y = np.r_[Y]

    IR = IsotonicRegression(increasing=False, out_of_bounds='clip')
    IR.fit(Xi, Y)
    Y1 = IR.predict(Xi)
    filter_X = []
    filter_Y = []
    for i in range(len(Y1)):
        if Y1[i] not in filter_Y:
            filter_Y.append(Y1[i])
            filter_X.append([X[i]])

    reg = LinearRegression()
    reg.fit(filter_X, filter_Y)
    slope = reg.coef_[0]

    return slope


def scale_matrix(matrix, start1, end1, start2, end2, glob_exp):
    SV_exp = {}
    dist_count = {}
    for i in range(start1, end1):
        for j in range(start2, end2):
            dist = abs(j - i)
            if dist not in SV_exp:
                SV_exp[dist] = 0
                dist_count[dist] = 0
            SV_exp[dist] += matrix[i][j]
            dist_count[dist] += 1
    filter_exp = {}
    for key in SV_exp:
        if dist_count[key] > 2:
            filter_exp[key] = SV_exp[key] / dist_count[key]
        # SV_exp[key] /= dist_count[key]
    if filter_exp:
        slope = linear_regression(filter_exp, glob_exp)
        if slope > 0:
            for i in range(start1, end1):
                for j in range(start2, end2):
                    matrix[i][j] /= slope

    return matrix


def adjust_matrix(matrix, glob_exp, path, start, end):
    interval = []
    for i in range(len(path)):
        row_start = get_start(path[:i+1], start, end)
        if row_start not in interval:
            interval.append(row_start)
        for j in range(i+1, len(path)):
            col_start = get_start(path[:j+1], start, end)
            start1 = row_start
            end1 = row_start+(end[i]-start[i])
            start2 = col_start
            end2 = col_start+(end[j]-start[j])
            matrix = scale_matrix(matrix, start1, end1, start2, end2, glob_exp)

    return matrix, interval


def read_one_assembly(filaname, resolution):
    path = []
    orient = []
    chrom = []
    start = []
    end = []
    with open(filaname) as f:
        f.readline()
        for line in f:
            line_data = line.strip().split('\t')
            chrom.append(line_data[0])
            start.append(int(line_data[1]) // resolution)
            end.append(int(line_data[2]) // resolution)
            path.append(line_data[3])
            orient.append(line_data[4])

    return path, orient, chrom, start, end


def Calculating_diagonal_data(matrix):
    """
    Calculating each diagonal mean
    """
    exp = {}
    N, M = len(matrix), len(matrix[0])
    for d in range(N):
        intermediate = []
        c = d
        r = 0
        while r < N - d:
            intermediate.append(matrix[r][c])
            r += 1
            c += 1
        intermediate = np.array(intermediate)
        exp[d] = np.mean(intermediate)
    return exp


def global_expect(dict_matrix, chrom_list):
    chrom = chrom_list[0] + '_' + chrom_list[0]
    matrix = dict_matrix[chrom]
    global_exp = Calculating_diagonal_data(matrix)
    IR = IsotonicRegression(increasing=False, out_of_bounds='clip')
    _d = np.r_[sorted(global_exp)]
    _y = np.r_[[global_exp[i] for i in _d]]
    IR.fit(_d, _y)
    d = np.arange(500 + 1)
    exp = IR.predict(d)
    expected = dict(zip(d, exp))
    return expected


# running HiSV
import cooler
import numpy as np
import numpy as np
import math
from itertools import groupby
from collections import defaultdict
from skimage.restoration import denoise_tv_chambolle

def combine(lst):
    # Merge consecutive identical values
    pos = (j - i for i, j in enumerate(lst))
    t = 0
    for i, els in groupby(pos):
        l = len(list(els))
        el = lst[t]
        t += l
        yield range(el, el+l)


def group(data):
    '''
    # Merge consecutive identical values
    :param data: pos index
    :return: start and end of consecutive same values
    '''
    last = data[0]
    start = end = 0
    for n in data[1:]:
        if n - last <= 5: # Part of the group, bump the end
            last = n
            end += 1
        else: # Not part of the group, yield current group and start a new
            yield range(data[start], data[end]+1)
            last = n
            start = end = end + 1
    # yield start, end
    yield range(data[start], data[end]+1)



def group_position(pos1list, pos2list, binsize):
    result = defaultdict(list)
    l1 = sorted(set(pos1list), key=pos1list.index)
    pos1group = list(group(l1))
    for i in range(len(pos1group)):
        start_pos = pos1list.index(pos1group[i][0])
        end_pos = len(pos1list) - 1 - pos1list[::-1].index(pos1group[i][-1])
        cur_pos2_list = pos2list[start_pos:end_pos + 1]
        cur_pos1_list = pos1list[start_pos:end_pos + 1]
        cur_pos2_list_sort = sorted(cur_pos2_list)
        l2 = sorted(set(cur_pos2_list_sort), key=cur_pos2_list_sort.index)
        pos2group = list(group(l2))
        if len(pos2group) > 1:
            for j in range(len(pos2group)):
                new_pos1 = []
                for k in range(len(pos2group[j])):
                    if pos2group[j][k] in cur_pos2_list:
                        pos = cur_pos2_list.index(pos2group[j][k])
                        new_pos1.append(cur_pos1_list[pos])        
                new_pos1 = sorted(new_pos1)
                result['pos1_start'].append(new_pos1[0] * binsize)
                result['pos1_end'].append(new_pos1[-1] * binsize + binsize)
                result['pos2_start'].append(pos2group[j][0] * binsize)
                result['pos2_end'].append(pos2group[j][-1] * binsize + binsize)

        else:
            result['pos1_start'].append(pos1group[i][0] * binsize)
            result['pos1_end'].append(pos1group[i][-1] * binsize + binsize)
            result['pos2_start'].append(pos2group[0][0] * binsize)
            result['pos2_end'].append(pos2group[0][-1] * binsize + binsize)
    return result


def load_matrix(clr, chrom1, chrom2):
    # load matrix from cool file (normalized matrix)
    if chrom1 == chrom2:
        M = clr.matrix(balance=False).fetch(chrom1)
    else:
        M = clr.matrix(balance=False).fetch(chrom1, chrom2)
    M[np.isnan(M)] = 0
    return M
    
def cool2mat(clr, chrom1, chrom2):
    chromsomes = clr.chromnames
    if chrom1 not in chromsomes or chrom2 not in chromsomes:
        raise ValueError(f"Chromosome number is invalidation.")
    else:
        M = load_matrix(clr, chrom1, chrom2)
    return M


def local_saliency(matrix, win):
    """
    # calculate local saliency map
    :param matrix: current Hi-C contact matrix
    :param win: 'Local region window.
    :return: saliency map
    """
    d = int(win / 2)
    N, M = len(matrix), len(matrix[0])
    nm_matrix = np.full((N, M), 0.0)
    Dist_matrix = np.full((win, win), 0.0)

    for m in range(win):
        for n in range(win):
            Dist_matrix[m][n] = np.sqrt(np.square(m) + np.square(n))

    it = np.nditer(matrix, flags=['multi_index'])
    while not it.finished:
        idx = it.multi_index
        data = matrix[idx]
        if data != 0:
            cur_matrix = matrix[idx[0] - d:idx[0] + d, idx[1] - d:idx[1] + d]
            cur_raw = cur_matrix.shape[0]
            cur_col = cur_matrix.shape[1]
            if cur_col == cur_raw == win and np.sum(cur_matrix) != 0:
                nm_matrix[idx] = 1 - math.exp(-np.mean(abs(data - cur_matrix) / (1 + Dist_matrix)))
            else:
                nm_matrix[idx] = 0
        it.iternext()
    return nm_matrix


def Calculating_diagonal_data(matrix):
    """
    # normalization matrix by diagonal to remove distance bias
    Calculating each diagonal mean and std
    """
    N, M = len(matrix), len(matrix[0])
    Diagonal_mean = np.full(M, 0.0)
    Diagonal_std = np.full(M, 0.0)
    std = []
    for d in range(N):
        intermediate = []
        c = d
        r = 0
        while r < N - d:
            intermediate.append(matrix[r][c])
            r += 1
            c += 1
        intermediate = np.array(intermediate)
        Diagonal_mean[d] = (np.mean(intermediate))
        Diagonal_std[d] = (np.std(intermediate))
    return Diagonal_mean, Diagonal_std


def Distance_normalization(matrix):
    """
    # normalization matrix by diagonal to remove distance bias
    norm_data = (data - mean_data) / mean_std
    """
    Diagonal_mean, Diagonal_std = Calculating_diagonal_data(matrix)
    N, M = len(matrix), len(matrix[0])
    for d in range(N):
        c = d
        r = 0
        while r < N - d:
            if Diagonal_std[d] == 0:
                matrix[r][c] = 0
            else:
                if matrix[r][c] - Diagonal_mean[d] < 0:
                    matrix[r][c] = 0
                else:
                    matrix[r][c] = (matrix[r][c] - Diagonal_mean[d]) / Diagonal_std[d]
            r += 1
            c += 1
    return matrix


def HiSV(clr, chromList, win, weight, cutoff):
    # load matrix from cool file
    combine_result = defaultdict(list)
    resolution = clr.info['bin-size']
    for i in range(len(chromList)):
        for j in range(i, len(chromList)):
            chrom1 = chromList[i]
            chrom2 = chromList[j]
            mat = cool2mat(clr, chrom1, chrom2)
            # running HiSV
            pos1 = []
            pos2 = []
            # filter low reads
            if chrom1 == chrom2:
            # intra-chromosomes SV
                # filter low reads
                non_zero_rows_and_cols = np.where(np.any(mat != 0, axis=1))[0]
                mat = mat[non_zero_rows_and_cols][:, non_zero_rows_and_cols]
                num = np.shape(mat)[0]
                il = np.tril_indices(num)
                mat[il] = 0
                # z-score for distance normalization
                Dist_norm_mat = Distance_normalization(mat)
                # calculating local saliency
                local_sali_matrix = local_saliency(Dist_norm_mat, win)
                # Segmentation
                seg_mat = denoise_tv_chambolle(local_sali_matrix, weight=weight)
                # Filter
                for m in range(num):
                    for n in range(num):
                        if seg_mat[m][n] > cutoff:
                            pos1.append(non_zero_rows_and_cols[m])
                            pos2.append(non_zero_rows_and_cols[n])
                result = group_position(pos1, pos2, resolution)
            else:
                # inter-chromosomes SV
                # filter
                non_zero_rows = np.where(np.any(mat != 0, axis=1))[0]
                non_zero_cols = np.where(np.any(mat != 0, axis=0))[0]
                mat = mat[non_zero_rows][:, non_zero_cols]
                raw_num, col_num = np.shape(mat)
                # calculating local saliency
                local_sali_matrix = local_saliency(mat, win)
                # Segmentation
                seg_mat = denoise_tv_chambolle(local_sali_matrix, weight=0.2)
                # Filter
                for m in range(raw_num):
                    for n in range(col_num):
                        if seg_mat[m][n] > cutoff:
                            pos1.append(non_zero_rows[m])
                            pos2.append(non_zero_cols[n])
                result = group_position(pos1, pos2, resolution)
            result['chrom1'] = [chrom1] * len(result['pos1_start'])
            result['chrom2'] = [chrom2] * len(result['pos2_start']) 
            for key, values in result.items():
                combine_result[key].extend(values)
    return dict(combine_result)



import os
from matplotlib.colors import LinearSegmentedColormap
import numpy as np
import operator
import matplotlib.pyplot as plt


def load_matrix(clr, chr_pair):
    M = clr.matrix(balance=False).fetch(chr_pair[0], chr_pair[1])
    M[np.isnan(M)] = 0
    return M

def load_norm_matrix(clr, chr_pair):
    M = clr.matrix(balance='sweight').fetch(chr_pair[0], chr_pair[1])
    M[np.isnan(M)] = 0
    return M

def load_bias(clr, chr_pair):
    bias = clr.bins().fetch(chr_pair[0])['sweight'].values
    # bias = clr.matrix(balance='sweight').fetch(chr_pair[0], chr_pair[1])
    bias[np.isnan(bias)] = 0
    return bias

def get_MatrixFile(hic_path, name):
    matrix = 0
    for filewalks in os.walk(hic_path):
        for files in filewalks[2]:
            if name in files:
                matrix = np.loadtxt(os.path.join(filewalks[0], files))
    return matrix



def runFitHiC(path, control_dict, chrom_list, chro_lens, resolution):
    with open(path + '/tmp/fithic.bed', 'w+') as f:
        chr_first = {}
        index = 1
        for idx, chro_len in enumerate(chro_lens):
            chr = chrom_list[idx]
            chr_first[chr] = index
            start = 0
            for _ in range(chro_len):
                f.write(chr + '\t' + str(start) + '\t' + str(start + resolution) + '\t' + str(index) + '\n')
                start += resolution
                index += 1
    with open(path + '/tmp/fithic.matrix', 'w+') as f:
        for i in range(len(chrom_list)):
            for j in range(i, len(chrom_list)):
                chr1_first = chr_first[chrom_list[i]]
                chr2_first = chr_first[chrom_list[j]]
                name = str(chrom_list[i]) + '_' + str(chrom_list[j])
                cur_matrix = control_dict[name]
                for bin_i in range(cur_matrix.shape[0]):
                    for bin_j in range(bin_i, cur_matrix.shape[1]):
                        if cur_matrix[bin_i][bin_j] > 0:
                            f.write(str(chr1_first + bin_i) + '\t' + str(chr2_first + bin_j) + '\t' + str(cur_matrix[bin_i][bin_j]) + '\n')
    current_path = os.path.dirname(os.path.abspath(__file__))
    fithic_path = os.path.join(current_path, '../decgr/fithic')

    os.system('python {0}/HiCPro2FitHiC.py -i {2} -b {3} -o {1}/tmp -r {4}'.format(
        fithic_path, path, path + '/tmp/fithic.matrix', path + '/tmp/fithic.bed', str(resolution)))
    os.system(
        'python {0}/fithic_copy.py -i {1}/tmp/fithic.interactionCounts.gz -f {1}/tmp/fithic.fragmentMappability.gz -o {1}/tmp -r {2} -x All'
        .format(fithic_path, path, str(resolution)))

    return path + '/result/dis2cnt'





##############################################################################
### To use the functions in this lib simply import this python module using
### import myStats
### Then you'll able able to call functions with the proper arguments using
### returnVal=myStats.func1(arg1,arg2)
##############################################################################
##############################################################################
import numpy as np

################### FUNC benjamini_hochberg_correction  #####################
#### Given an array of p-values (not necessarily sorted) and the number of total 
### tests that were performed to gather these p-values, this function performs
###  the multiple hypothesis testing correction described by Benjamini-Hochberg.
###
### 
### If the number of tests are much more compared to p-value array and
### the omitted p-values are all supposed to be zero you should use it like:
### q_array=benjamini_hochberg_correction([0.03,0.4,0.7,0.01],10)
### 
### If the number of tests are equal to the ones in p-values array then:
### p_array=[0.03,0.4,0.7,0.01]
### q_array=benjamini_hochberg_correction(p_array,len(p_array))
##############################################################################
def benjamini_hochberg_correction(p_values, num_total_tests):
	# assumes that p_values vector might not be sorted
	pvalsArray=np.array(p_values)
	order=pvalsArray.argsort()
	sorted_pvals=np.take(p_values,order)
	q_values=[1.0 for i in range(len(p_values))]
	prev_bh_value = 0
	for i, p_value in enumerate(sorted_pvals):
		if p_value==1.0:
			bh_value=1.0
		else:
			bh_value = p_value * num_total_tests / (i + 1)
			# Sometimes this correction can give values greater than 1,
			# so we set those values at 1
			bh_value = min(bh_value, 1)
		#
		# To preserve monotonicity in the values, we take the
		# maximum of the previous value or this one, so that we
		# don't yield a value less than the previous.
		bh_value = max(bh_value, prev_bh_value)
		prev_bh_value = bh_value
		qIndex=order[i]
		q_values[qIndex] =bh_value
	#END for
	return q_values

################### FUNC meanAndVariance  ##################################
#### Given an array this function calculates the mean and variance 
##############################################################################
def meanAndVariance(a):
	#calculates variance using E(x^2) - (Ex)^2
	sumxsquared=0
	sumx=0
	for x in a:
		sumxsquared += (x*x)
		sumx += x
	mean=sumx/float(len(a))
	squaremean=sumxsquared/float(len(a))
	variance=squaremean - (mean*mean)
	return (mean, variance)




#cython: language_level=3
#cython: boundscheck=False
#cython: cdivision=True

import os, cooler, bisect, h5py
import numpy as np
from cooler import util
from scipy.stats import rankdata, mode

def find_chrom_pre(chromlabels):

    ini = chromlabels[0]
    if ini.startswith('chr'):
        return 'chr'
    
    else:
        return ''

# load CNV of any resolutions into a cool URI at specific resolution

class binCNV(object):

    def __init__(self, bedgraph, res):

        self.binsize = res

        ref_pre = ''

        self.load_bedGraph(bedgraph)
        ori_pre = find_chrom_pre(list(self.cnv_segment))
        D = {}
        for c in self.cnv_segment:
            ref_k = ref_pre+c.lstrip(ori_pre)
            chromlen = self.cnv_segment[c][-1][1]
            arr = []
            if self.cnv_res == res:
                for s, e, v in self.cnv_segment[c]:
                    arr.append(v)
            else:
                for i in range(0, chromlen, res):
                    start = i
                    end = min(i+res, chromlen)
                    tmp = self.calculate_bin(self.cnv_segment[c], [start, end])
                    arr.append(tmp)
            arr = np.r_[arr]
            D[ref_k] = arr
        
        # average copy numbers
        self.bin_cnv = D

    def calculate_bin(self, bychrom, interval):

        base_pairs = np.zeros(interval[1] - interval[0])
        idx = max(0, bisect.bisect(bychrom, interval)-1)
        for q in bychrom[idx:]:
            if q[1] <= interval[0]:
                continue
            if q[0] >= interval[1]:
                break
            s = q[0]-interval[0]
            if s < 0:
                s = 0
            e = q[1]-interval[0]
            base_pairs[s:e] += q[2]
        
        L = base_pairs.size
        if base_pairs[:L//2].mean() == base_pairs[L//2:].mean():
            return base_pairs.mean()
        else:
            return mode(base_pairs)[0][0]
    
    
    def load_bedGraph(self, infil):

        self.cnv_segment = {}
        with open(infil, 'r') as source:
            for line in source:
                p = line.rstrip().split()
                if p[0] in self.cnv_segment:
                    self.cnv_segment[p[0]].append([int(p[1]), int(p[2]), float(p[3])])
                else:
                    self.cnv_segment[p[0]] = [[int(p[1]), int(p[2]), float(p[3])]]
                    self.cnv_res = int(p[2]) - int(p[1])
        
        for c in self.cnv_segment:
            self.cnv_segment[c].sort()
        
    
    def assign_cnv(self, cooler_uri):
        
        cooler_lib = cooler.Cooler(cooler_uri)
        ref_pre = find_chrom_pre(cooler_lib.chromnames)
        cnv = np.r_[[]]
        for ref_k in cooler_lib.chromnames: # line with bin table
            bias = cooler_lib.bins().fetch(ref_k)['weight'].values
            c = ref_k.lstrip(ref_pre)
            if not c in self.bin_cnv:
                pre = np.zeros(len(bias))
                cnv = np.r_[cnv, pre]
                continue

            pre = self.bin_cnv[c]
            if len(bias) <= pre.size:
                pre = pre[:len(bias)]
            else:
                add = np.zeros(len(bias)-pre.size)
                pre = np.r_[pre, add]
            
            mask = np.isnan(bias) | (bias==0)
            pre[mask] = 0

            cnv = np.r_[cnv, pre]
        
        cnvi = rankdata(cnv, method='dense') - 1 # indices for quick Bias retrival
        
        # pre-check the CNV column
        cool_path, group_path = util.parse_cooler_uri(cooler_uri)
        with h5py.File(cool_path, 'r+') as h5:
            grp = h5[group_path]
            if 'CNV' in grp['bins']:
                del grp['bins']['CNV'] # Overwrite the CNV column
                del grp['bins']['CNVI']
        
        with h5py.File(cool_path, 'r+') as h5:
            grp = h5[group_path]
            h5opts = dict(compression='gzip', compression_opts=6)
            grp['bins'].create_dataset('CNV', data=cnv, **h5opts)
            grp['bins'].create_dataset('CNVI', data=cnvi, dtype='i1', **h5opts)

        del cnv, cnvi


#!/usr/bin/env python

import numpy as np
import matplotlib.pyplot as plt
from matplotlib.colors import LinearSegmentedColormap
import pandas as pd
from collections import  defaultdict
from PyQt5.QtCore import QTimer
from PyQt5.QtWidgets import QPushButton, QVBoxLayout, QHBoxLayout, QMessageBox, QWidget, QSizePolicy
from matplotlib.backends.backend_qt5agg import FigureCanvasQTAgg as FigureCanvas
import bisect
import matplotlib.patches as patches
from matplotlib import gridspec

class MyFigureCanvas(FigureCanvas):
    def __init__(self, width, height, dpi, matrix, interval, assembly_dict, chrom_list, dict_start, dict_end, resolution, output_widget):
        self.fig = plt.figure(figsize=(width, height), dpi=dpi)
        super(MyFigureCanvas, self).__init__(self.fig)
        self.ax = self.fig.add_subplot(111)
        self.data = matrix
        self.vmax = np.percentile(matrix, 98)
        self.vmin = matrix.min()
        cdict = {
            'red': ((0.0, 0.0, 1.0), (1.0, 1.0, 1.0)),
            'green': ((0.0, 0.0, 1.0), (1.0, 0.0, 0.0)),
            'blue': ((0.0, 0.0, 1.0), (1.0, 0.0, 0.0))
        }
        self.cmap = LinearSegmentedColormap('Rd_Bl_Rd', cdict, 256)
        self.interval = interval
        self.chrom_list = chrom_list
        self.dict_start = dict_start
        self.resolution = resolution
        self.output_widget = output_widget
        self.image = self.ax.imshow(matrix, cmap=self.cmap, vmin=self.vmin, vmax=self.vmax)
        row = matrix.shape[0]
        self.ax.text(row / 2, row + 30, "Original", 
                horizontalalignment='center', verticalalignment='bottom', 
                fontsize=14, color="black", fontweight="bold")
        self.ax.text(row / 2, -30, "Simulated", 
                horizontalalignment='center', verticalalignment='top', 
                fontsize=14, color="black", fontweight="bold")
        # plot chrom boundary
        for chrom, cur_chrom_start in interval.items():
            info = str(chrom) + ":" + str(dict_start[chrom] * self.resolution // 1000000) + '-' + str(dict_end[chrom] * self.resolution // 1000000) + 'M'
            self.ax.text(cur_chrom_start, -10, f'{info}', horizontalalignment='center', color='black', fontsize=12)
            if cur_chrom_start != 0:
                self.ax.plot([0, row], [cur_chrom_start, cur_chrom_start],
                        color="black", linestyle='--', linewidth=2)
                self.ax.plot([cur_chrom_start, cur_chrom_start], [0, row],
                        color="black", linestyle='--', linewidth=2)
        # plot matrix boundary       
        self.ax.plot([0, row], [0, row], color='black', linewidth=2)
        self.ax.plot([0, 0], [0, row], color='black', linewidth=2)
        self.ax.plot([0, row], [0, 0], color='black', linewidth=2)
        self.ax.plot([row, row], [row, 0], color='black', linewidth=2)
        self.ax.plot([row, 0], [row, row], color='black', linewidth=2)
        # plot fragment boundary
        for num, group_df in assembly_dict.items():
            row_pair = []
            for i in range(len(group_df) - 1):
                row1 = group_df.iloc[i]
                row2 = group_df.iloc[i + 1]
                if row1['start'] <= row2['start']:
                    x1 = row1['start'] + interval[row1['chrom']]
                    x2 = row1['end'] + interval[row1['chrom']]
                    y1 = row2['start'] + interval[row2['chrom']]
                    y2 = row2['end'] + interval[row2['chrom']]
                else:
                    y1 = row1['start'] + interval[row1['chrom']]
                    y2 = row1['end'] + interval[row1['chrom']]
                    x1 = row2['start'] + interval[row2['chrom']]
                    x2 = row2['end'] + interval[row2['chrom']]
                info = str(row1['node']) + '-' + str(row2['node'])
                self.ax.text(x1, y1, f'{info}', fontsize=10, verticalalignment='bottom', horizontalalignment='right', color='black')
                self.ax.plot([y1, x1], [y1, y1], color="black", linestyle='--', linewidth=0.5)
                self.ax.plot([x1, x1], [x1, y2], color="black", linestyle='--', linewidth=0.5)
                self.ax.plot([y2, x1], [y2, y2], color="black", linestyle='--', linewidth=0.5)
                self.ax.plot([x2, x2], [x2, y2], color="black", linestyle='--', linewidth=0.5)

                self.ax.plot([y1, y1], [y1, x1], color="black", linestyle='--', linewidth=0.5)
                self.ax.plot([y2, y2], [y2, x1], color="black", linestyle='--', linewidth=0.5)
                self.ax.plot([x1, y2], [x1, x1], color="black", linestyle='--', linewidth=0.5)
                self.ax.plot([x2, y2], [x2, x2], color="black", linestyle='--', linewidth=0.5)
                self.ax.scatter([x1, x2, x1, x2], [y1, y1, y2, y2], color="black", s=5)
                self.ax.scatter([y1, y1, y2, y2], [x1, x2, x1, x2], color="black", s=5)
        self.ax.axis('off')
        # mouse_connect
        self.mpl_connect('scroll_event', self.on_scroll)
        self.mpl_connect('button_press_event', self.on_press)
        self.mpl_connect('motion_notify_event', self.on_move)
        self.mpl_connect('button_release_event', self.on_release)
        self.mpl_connect('button_press_event', self.on_click)
        # set press
        self.press = None
        self.draw()

    def on_scroll(self, event):
        x_left, x_right = self.ax.get_xlim()
        y_bottom, y_top = self.ax.get_ylim()
        # set zoom factor
        zoom_factor = 1.2

        if event.button == 'up':
            scale_factor = 1 / zoom_factor
        elif event.button == 'down':
            scale_factor = zoom_factor
        else:
            scale_factor = 1

        xdata, ydata = event.xdata, event.ydata
        new_width = (x_right - x_left) * scale_factor
        new_height = (y_top - y_bottom) * scale_factor

        x_left = xdata - (xdata - x_left) * scale_factor
        x_right = xdata + (x_right - xdata) * scale_factor
        y_bottom = ydata - (ydata - y_bottom) * scale_factor
        y_top = ydata + (y_top - ydata) * scale_factor

        self.ax.set_xlim([x_left, x_right])
        self.ax.set_ylim([y_bottom, y_top])
        self.draw()

    def on_press(self, event):
        if event.button == 1:
            if event.inaxes != self.ax:
                return
            self.press = (event.xdata, event.ydata)
            if event.dblclick:
                self.on_click(event) 

    def on_move(self, event):
        if self.press is None:
            return
        if event.xdata is None or event.ydata is None:
            return
        x_left, x_right = self.ax.get_xlim()
        y_bottom, y_top = self.ax.get_ylim()
        dx = event.xdata - self.press[0]
        dy = event.ydata - self.press[1]
        self.ax.set_xlim(x_left - dx, x_right - dx)
        self.ax.set_ylim(y_bottom - dy, y_top - dy)
        self.press = (event.xdata, event.ydata)
        self.draw()

    def on_release(self, event):
        self.press = None

    def on_click(self, event):
        if event.inaxes != self.ax:
            return
        x_click = int(event.xdata)
        y_click = int(event.ydata)
        edges = sorted(self.interval.values())
        chromosomes = sorted(self.interval, key=self.interval.get)
        x_chr_idx = bisect.bisect_right(edges, x_click) - 1
        y_chr_idx = bisect.bisect_right(edges, y_click) - 1
        if x_chr_idx == y_chr_idx and 0 <= x_chr_idx < len(chromosomes):
            chr_clicked = chromosomes[x_chr_idx]
            start_pos = self.dict_start[chr_clicked]
            pixel_value = self.data[y_click, x_click]
            final_position_x = (start_pos + (x_click - self.interval[chr_clicked])) * self.resolution
            final_position_y = (start_pos + (y_click - self.interval[chr_clicked])) * self.resolution
            output_text = str(chr_clicked) + ":" + str(final_position_x) + "-" + str(final_position_y)
        elif 0 <= x_chr_idx < len(chromosomes) and 0 <= y_chr_idx < len(chromosomes):
            chr_x = chromosomes[x_chr_idx]
            chr_y = chromosomes[y_chr_idx]
            pixel_value = self.data[y_click, x_click]
            final_position_x_in_chr_x = (self.dict_start[chr_x] + (x_click - self.interval[chr_x])) * self.resolution
            final_position_y_in_chr_y = (self.dict_start[chr_y] + (y_click - self.interval[chr_y])) * self.resolution
            output_text = str(chr_x) + ":" + str(final_position_x_in_chr_x) + "," + str(chr_y) + ":" + str(final_position_y_in_chr_y)
        else:
            output_text = "Error position!"
        self.output_widget.setText(f"Clicked at ({output_text}), Count: {pixel_value:.3f}")
       

def simuCount(matrix_dict, dis2cnt, res, genome, SV_chrom):
    d2c = {}
    with open(dis2cnt, 'r') as f:
        for line in f:
            dat = line.split()
            if dat[0][0] == '-':
                d2c['inter'] = float(dat[1])
            else:
                d2c[int(int(dat[0]) / res)] = float(dat[1])
    d2cLen = len(d2c) - 2
    SV_matrix = {}
    for key in matrix_dict:
        row, col = np.shape(matrix_dict[key])
        SV_matrix[key] = np.full((row, col), 0.0)
    for i in range(len(genome)):
        # for j in range(i+1, len(genome)):
        for j in range(len(genome)):
            # new_dist = abs(genome.index(j) - genome.index(i))
            if SV_chrom[i] == SV_chrom[j]:
                order_dist = abs(genome[i] - genome[j])
                pos1 = min(genome[i], genome[j])
                pos2 = max(genome[i], genome[j])
            else:
                order_dist = d2cLen
                pos1 = genome[i]
                pos2 = genome[j]

            new_dist = abs(i - j)
            scale = d2c[min(new_dist, d2cLen)] / d2c[min(order_dist, d2cLen)]
            if i < j:
                chrom_pair = SV_chrom[i] + '_' + SV_chrom[j]
            else:
                chrom_pair = SV_chrom[j] + '_' + SV_chrom[i]
            SV_matrix[chrom_pair][pos1, pos2] += matrix_dict[chrom_pair][pos1, pos2] * scale
    return SV_matrix



def simulation(assembly_dict, dis2cnt, control_dict, chrom_list, res):
    simu_dict = {}
    # initial matrix
    for i in range(len(chrom_list)):
        for j in range(i, len(chrom_list)):
            name = str(chrom_list[i]) + '_' + str(chrom_list[j])
            simu_dict[name] = np.full(np.shape(control_dict[name]), 0.0)
   
    for num, group_df in assembly_dict.items():
        genome = []
        SV_chrom = []
        for index, row in group_df.iterrows():            
            if row['orient'] == '+':
                fragment = list(range(row['start'], row['end']))
                cur_chrom = [row['chrom']] * (row['end'] - row['start'])
                genome.extend(fragment)
                SV_chrom.extend(cur_chrom)
            else:
                fragment = list(range(row['end'], row['start'], -1))
                cur_chrom = [row['chrom']] * (row['end'] - row['start'])
                SV_chrom.extend(cur_chrom)
                genome.extend(fragment)
            cur_simu_dict = simuCount(control_dict, dis2cnt, res, genome, SV_chrom)
            for key in cur_simu_dict:
                simu_dict[key] += cur_simu_dict[key]

    for key in simu_dict:
        simu_matrix = simu_dict[key]
        for i in range(len(simu_matrix[0])):
            for j in range(len(simu_matrix[0])):
                if simu_matrix[i, j] == 0:
                    simu_dict[key][i, j] = control_dict[key][i, j]

    return simu_dict


def combine_matrix(dict_matrix, dict_start, dict_end, chrom_list):
    # init_matrix
    sum_bin = 0
    for i in range(len(chrom_list)):
        sum_bin += (dict_end[chrom_list[i]] - dict_start[chrom_list[i]])
    all_matrix = np.full((sum_bin, sum_bin), 0)
    chrom_1 = 0
    interval = {}
    for i in range(len(chrom_list)):
        start1 = dict_start[chrom_list[i]]
        end1 = dict_end[chrom_list[i]]
        length1 = end1 - start1
        chrom_2 = chrom_1
        interval[chrom_list[i]] = chrom_1
        for j in range(i, len(chrom_list)):
            name = str(chrom_list[i]) + '_' + str(chrom_list[j])
            start2 = dict_start[chrom_list[j]]
            end2 = dict_end[chrom_list[j]]
            length2 = end2 - start2
            if j == i:
                cur_matrix = dict_matrix[name]
                cur_matrix = np.triu(cur_matrix)
                cur_matrix += cur_matrix.T - np.diag(cur_matrix.diagonal())
                dict_matrix[name] = cur_matrix
            all_matrix[chrom_1:chrom_1+length1, chrom_2:chrom_2+length2] = dict_matrix[name][start1:end1, start2:end2]
            chrom_2 += length2
        chrom_1 += length1
    return all_matrix, interval


def get_cut_pos(assembly_dict, resolution, dict_count):
    dict_start = {}
    dict_end = {}
    for num, group_df in assembly_dict.items():
        for index, row in group_df.iterrows():
            cur_start = max(0, row['start']-5000000//resolution)
            cur_end = min(row['end'] + 5000000 // resolution, dict_count[row['chrom']])
            if row['chrom'] in dict_start and dict_start[row['chrom']] is not None:
                dict_start[row['chrom']] = min(cur_start, dict_start[row['chrom']])
                dict_end[row['chrom']] = max(cur_end, dict_end[row['chrom']])
            else:
                dict_start[row['chrom']] = cur_start
                dict_end[row['chrom']] = cur_end
    return dict_start, dict_end


def plot_simu_matrix(dict_matrix, simu_dict, dict_start, dict_end, chrom_list, assembly_dict, resolution, output_widget):
    for num, group_df in assembly_dict.items():
        for index, row in group_df.iterrows():
            cur_chrom = row['chrom']
            assembly_dict[num].loc[index, 'start'] -= dict_start[cur_chrom]
            assembly_dict[num].loc[index, 'end'] -= dict_start[cur_chrom]
    original_matrix, interval = combine_matrix(dict_matrix, dict_start, dict_end, chrom_list)
    simu_matrix, interval = combine_matrix(simu_dict, dict_start, dict_end, chrom_list)
    upper = np.triu(simu_matrix)
    upper_indices = np.triu_indices(simu_matrix.shape[0])
    original_matrix[upper_indices] = upper[upper_indices]
    np.fill_diagonal(original_matrix, 0)
    canvas = MyFigureCanvas(width=5, height=4, dpi=100, matrix=original_matrix, interval=interval, resolution=resolution,
                            assembly_dict=assembly_dict, chrom_list=chrom_list, dict_start=dict_start, dict_end=dict_end, output_widget=output_widget)
    return canvas


def read_assembly_result(filaname, resolution):
    df = pd.read_csv(filaname, sep='\t')
    df[['start', 'end']] = df[['start', 'end']] // resolution
    assembly_dict = {num: group.drop(columns=['num']).reset_index(drop=True) for num, group in df.groupby('num')}
    return assembly_dict


